@@ -14,7 +14,7 @@ use Carp;
 use Time::Local;
 use Net::Config;
 
-$VERSION = "2.16"; # $Id: //depot/libnet/Net/NNTP.pm#4$
+$VERSION = "2.18"; # $Id: //depot/libnet/Net/NNTP.pm#8$
 @ISA     = qw(Net::Cmd IO::Socket::INET);
 
 sub new
@@ -61,19 +61,22 @@ sub new
  my $c = $obj->code;
  my @m = $obj->message;
  
- # if server is INN and we have transfer rights the we are currently
- # talking to innd not nnrpd
- if($obj->reader)
-  {
-   # If reader suceeds the we need to consider this code to determine postok
-   $c = $obj->code;
-  }
- else
-  {
-   # I want to ignore this failure, so restore the previous status.
-   $obj->set_status($c,\@m);
-  }
- ${*$obj}{'net_nntp_post'} = $c >= 200 && $c <= 209 ? 1 : 0;
+ unless(exists $arg{Reader} && $arg{Reader} == 0) {
+   # if server is INN and we have transfer rights the we are currently
+   # talking to innd not nnrpd
+   if($obj->reader)
+    {
+     # If reader suceeds the we need to consider this code to determine postok
+     $c = $obj->code;
+    }
+   else
+    {
+     # I want to ignore this failure, so restore the previous status.
+     $obj->set_status($c,\@m);
+    }
+ }
+
+ ${*$obj}{'net_nntp_post'} = $c == 200 ? 1 : 0;
 
  $obj;
 }
@@ -678,6 +681,12 @@ NNTP server, a value of zero will cause all IO operations to block.
 
 B<Debug> - Enable the printing of debugging information to STDERR
 
+B<Reader> - If the remote server is INN then initially the connection
+will be to nnrpd, by default C<Net::NNTP> will issue a C<MODE READER> command
+so that the remote server becomes innd. If the C<Reader> option is given
+with a value of zero, then this command will not be sent and the
+connection will be left talking to nnrpd.
+
 =back
 
 =head1 METHODS
@@ -956,7 +965,7 @@ second number in a range is less than or equal to the first then the range
 represents all messages in the group after the first message number.
 
 B<NOTE> For compatibility reasons only with earlier versions of Net::NNTP
-a message spec can be passed as a list of two numbers, this is depreciated
+a message spec can be passed as a list of two numbers, this is deprecated
 and a reference to the list should now be passed
 
 =item PATTERN
