@@ -16,7 +16,7 @@ use IO::Socket;
 use Net::Cmd;
 use Net::Config;
 
-$VERSION = "2.10"; # $Id: //depot/libnet/Net/SMTP.pm#4$
+$VERSION = "2.15"; # $Id: //depot/libnet/Net/SMTP.pm#11 $
 
 @ISA = qw(Net::Cmd IO::Socket::INET);
 
@@ -56,9 +56,14 @@ sub new
 
  ${*$obj}{'net_smtp_host'} = $host;
 
+ (${*$obj}{'net_smtp_banner'}) = $obj->message;
  (${*$obj}{'net_smtp_domain'}) = $obj->message =~ /\A\s*(\S+)/;
 
- $obj->hello($arg{Hello} || "");
+ unless($obj->hello($arg{Hello} || ""))
+  {
+   $obj->close();
+   return undef;
+  }
 
  $obj;
 }
@@ -67,6 +72,13 @@ sub new
 ## User interface methods
 ##
 
+sub banner
+{
+ my $me = shift;
+
+ return ${*$me}{'net_smtp_banner'} || undef;
+}
+
 sub domain
 {
  my $me = shift;
@@ -74,6 +86,12 @@ sub domain
  return ${*$me}{'net_smtp_domain'} || undef;
 }
 
+sub etrn {
+    my $self = shift;
+    defined($self->supports('ETRN',500,["Command unknown: 'ETRN'"])) &&
+	$self->_ETRN(@_);
+}
+
 sub hello
 {
  my $me = shift;
@@ -84,31 +102,38 @@ sub hello
 		   } ||
 		"";
  my $ok = $me->_EHLO($domain);
- my $msg;
+ my @msg = $me->message;
 
  if($ok)
   {
-   $msg = $me->message;
-
    my $h = ${*$me}{'net_smtp_esmtp'} = {};
-   my $ext;
-   foreach $ext (qw(8BITMIME CHECKPOINT DSN SIZE))
-    {
-     $h->{$ext} = 1
-	if $msg =~ /\b${ext}\b/;
+   my $ln;
+   foreach $ln (@msg) {
+     $h->{$1} = $2
+	if $ln =~ /(\S+)\b[ \t]*([^\n]*)/;
     }
   }
- else
+ elsif($me->status == CMD_ERROR) 
   {
-   $msg = $me->message
-	if $me->_HELO($domain);
+   @msg = $me->message
+	if $ok = $me->_HELO($domain);
   }
 
- $ok && $msg =~ /\A(\S+)/
+ $ok && $msg[0] =~ /\A(\S+)/
 	? $1
 	: undef;
 }
 
+sub supports {
+    my $self = shift;
+    my $cmd = uc shift;
+    return ${*$self}{'net_smtp_esmtp'}->{$cmd}
+	if exists ${*$self}{'net_smtp_esmtp'}->{$cmd};
+    $self->set_status(@_)
+	if @_;
+    return;
+}
+
 sub _addr
 {
  my $addr = shift || "";
@@ -231,14 +256,16 @@ sub reset
 sub recipient
 {
  my $smtp = shift;
- my $ok = 1;
  my $opts = "";
+ my $skip_bad = 0;
 
  if(@_ && ref($_[-1]))
   {
    my %opt = %{pop(@_)};
    my $v;
 
+   $skip_bad = delete $opt{'SkipBad'};
+
    if(exists ${*$smtp}{'net_smtp_esmtp'})
     {
      my $esmtp = ${*$smtp}{'net_smtp_esmtp'};
@@ -260,18 +287,25 @@ sub recipient
 		. ' - ignored'
 	if scalar keys %opt;
     }
-   else
+   elsif(%opt)
     {
      carp 'Net::SMTP::recipient: ESMTP not supported by host - options discarded :-(';
     }
   }
 
- while($ok && scalar(@_))
+ my @ok;
+ my $addr;
+ foreach $addr (@_) 
   {
-   $ok = $smtp->_RCPT("TO:" . _addr(shift) . $opts);
+    if($smtp->_RCPT("TO:" . _addr($addr) . $opts)) {
+      push(@ok,$addr) if $skip_bad;
+    }
+    elsif(!$skip_bad) {
+      return 0;
+    }
   }
 
- return $ok;
+ return $skip_bad ? @ok : 1;
 }
 
 sub to { shift->recipient(@_) }
@@ -315,8 +349,7 @@ sub quit
 
 sub DESTROY
 {
- my $me = shift;
- defined(fileno($me)) && $me->quit
+# ignore
 }
 
 ##
@@ -338,6 +371,7 @@ sub _NOOP { shift->command("NOOP")->response()	    == CMD_OK }
 sub _QUIT { shift->command("QUIT")->response()	    == CMD_OK }   
 sub _DATA { shift->command("DATA")->response()	    == CMD_MORE } 
 sub _TURN { shift->unsupported(@_); } 			   	  
+sub _ETRN { shift->command("ETRN", @_)->response()  == CMD_OK }
 
 1;
 
@@ -442,6 +476,11 @@ empty list.
 
 =over 4
 
+=item banner ()
+
+Returns the banner message which the server replied with when the
+initial connection was made.
+
 =item domain ()
 
 Returns the domain that the remote SMTP server identified itself as during
@@ -454,6 +493,10 @@ command (or HELO if EHLO fails).  Since this method is invoked
 automatically when the Net::SMTP object is constructed the user should
 normally not have to call it manually.
 
+=item etrn ( DOMAIN )
+
+Request a queue run for the DOMAIN given.
+
 =item mail ( ADDRESS [, OPTIONS] )
 
 =item send ( ADDRESS )
@@ -483,7 +526,7 @@ Reset the status of the server. This may be called after a message has been
 initiated, but before any data has been sent, to cancel the sending of the
 message.
 
-=item recipient ( ADDRESS [, ADDRESS [ ...]] )
+=item recipient ( ADDRESS [, ADDRESS [ ...]] [, OPTIONS ] )
 
 Notify the server that the current message should be sent to all of the
 addresses given. Each address is sent as a separate command to the server.
@@ -491,6 +534,16 @@ Should the sending of any address result in a failure then the
 process is aborted and a I<false> value is returned. It is up to the
 user to call C<reset> if they so desire.
 
+The C<recipient> method can some additional OPTIONS which is passed
+in hash like fashion, using key and value pairs.  Possible options are:
+
+ Notify    =>
+ SkipBad   => ignore bad addresses
+
+If C<SkipBad> is true the C<recipient> will not return an error when a
+bad address is encountered and it will return an array of addresses
+that did succeed.
+
 =item to ( ADDRESS [, ADDRESS [...]] )
 
 A synonym for C<recipient>.
